using System;
using System.Net.Http;
using System.Text;
using System.Security.Cryptography;
using System.Web;

class APIExample
{
    private const string API_KEY = "API_KEY";
    private const string API_SECRET = "API_SECRET";
    private const string IF_DATABASE_ID = "USED_FOR_FAR";

    private const string API_METHOD = "GET";
    private const string API_HOST = "https://faculty180.interfolio.com/api.php";
    private const string API_ENDPOINT = "/users";
    private const string API_QUERY_PARAMS = "?data=count";

    public static void Main()
    {
        Console.WriteLine("Running...");
        try
        {
            // Build timestamp to spec
            var timestamp = DateTime.UtcNow.ToString("yyyy-MM-dd HH:mm:ss");

            // Build the request string (exclude query string parameters)
            var requestString = API_ENDPOINT;
            Console.WriteLine($"requestString: {requestString}");

            // Build message to be signed
            var message = BuildMessage(timestamp, requestString);
            Console.WriteLine("message:\n" + message);

            // Generate signature using HMAC-SHA1
            var signature = GenerateSignature(message, API_SECRET);
            Console.WriteLine("signature: " + signature);

            // Build the Interfolio API authorization header
            var authHeader = BuildHeader(signature);
            Console.WriteLine("authHeader: " + authHeader);

            // Build URL
            var apiUrl = API_HOST + API_ENDPOINT + API_QUERY_PARAMS;
            Console.WriteLine("apiUrl: " + apiUrl);

            // Make API request
            var responseText = MakeApiRequest(apiUrl, timestamp, authHeader);
            Console.WriteLine("Response: " + responseText);
        }
        catch (Exception e)
        {
            Console.WriteLine("error: " + e.Message);
        }
        Console.WriteLine("...all done.");
    }

    private static string BuildMessage(string timestamp, string requestString)
    {
        return $"{API_METHOD}\n\n\n{timestamp}\n{requestString}";
    }

    private static string GenerateSignature(string message, string secret)
    {
        using (var hmacsha1 = new HMACSHA1(Encoding.UTF8.GetBytes(secret)))
        {
            byte[] hashBytes = hmacsha1.ComputeHash(Encoding.UTF8.GetBytes(message));
            return Convert.ToBase64String(hashBytes);
        }
    }

    private static string BuildHeader(string signature)
    {
        return $"INTF {API_KEY}:{signature}";
    }

    private static string MakeApiRequest(string apiUrl, string timestamp, string authHeader)
    {
        using (var client = new HttpClient())
        {
            client.DefaultRequestHeaders.Add("TimeStamp", timestamp);
            client.DefaultRequestHeaders.Add("Authorization", authHeader);
            client.DefaultRequestHeaders.Add("INTF-DatabaseID", IF_DATABASE_ID);

            var response = client.GetAsync(apiUrl).Result;
            return response.Content.ReadAsStringAsync().Result;
        }
    }
}
